@@ -44,12 +44,25 @@ typedef struct pollfd {
 	short	revents;
 } pollfd_t;
 
-#define	POLLIN		0x0001
-#define	POLLPRI		0x0002
-#define	POLLOUT		0x0004
-#define	POLLERR		0x0008
-#define	POLLHUP		0x0010
-#define	POLLNVAL	0x0020
+#ifndef POLLIN
+# define POLLIN		0x0001
+#endif
+#ifndef POLLPRI
+# define POLLPRI	0x0002
+#endif
+#ifndef POLLOUT
+# define POLLOUT	0x0004
+#endif
+#ifndef POLLERR
+# define POLLERR	0x0008
+#endif
+#ifndef POLLHUP
+# define POLLHUP	0x0010
+#endif
+#ifndef POLLNVAL
+# define POLLNVAL	0x0020
+#endif
+
 #if 0
 /* the following are currently not implemented */
 #define	POLLRDNORM	0x0040
